from io import StringIO
from textwrap import dedent

import nbformat

from tex2ipy.tex2cells import Tex2Cells
from tex2ipy.cli import tex2ipy, main, get_tex2cells_subclass


DOCUMENT = dedent(r"""
\documentclass[14pt, compress]{beamer}
\begin{document}
\begin{frame}
\frametitle{Foo}
Hello world
\end{frame}
\end{document}
""")


def test_tex2ipy_should_make_notebook():
    # Given
    # When
    nb = tex2ipy(DOCUMENT)

    # Then
    cells = nb['cells']
    assert len(cells) == 1
    src = cells[0]['source']
    assert src[0] == '## Foo\n'
    assert src[1] == 'Hello world\n'

    md = nb['metadata']
    assert sorted(md.keys()) == ['celltoolbar', 'language', 'livereveal']


def test_get_tex2cells_subclass():
    # Given
    code = dedent("""
    from tex2ipy.tex2cells import Tex2Cells
    class MyConverter(Tex2Cells):
        pass
    """)
    stream = StringIO(code)

    # When
    cls = get_tex2cells_subclass(stream, 'test.py')

    # Then
    assert issubclass(cls, Tex2Cells)
    assert cls.__name__ == 'MyConverter'
    assert cls != Tex2Cells


def test_get_tex2cells_subclass_with_no_code_returns_none():
    # Given
    stream = StringIO('')

    # When
    cls = get_tex2cells_subclass(stream, 'test.py')

    # Then
    assert cls is None


def test_main_with_no_options(tmpdir):
    # Given
    src = tmpdir.join('test.tex')
    src.write(DOCUMENT)
    dest = tmpdir.join('test.ipynb')

    # When
    main(args=[str(src), str(dest)])

    # Then
    assert dest.check(file=1)


def test_main_with_options(tmpdir):
    # Given
    code = dedent("""
    from tex2ipy.tex2cells import Tex2Cells
    class Converter(Tex2Cells):
        def _handle_frame(self, node):
            super(Converter, self)._handle_frame(node)
            self.current['source'].append('## Overloaded\\n')
    """)
    convert = tmpdir.join('extra.py')
    convert.write(code)
    src = tmpdir.join('test.tex')
    src.write(DOCUMENT)
    dest = tmpdir.join('test.ipynb')

    # When
    main(args=[str(src), str(dest), '-c', str(convert)])

    # Then
    assert dest.check(file=1)
    nb = nbformat.read(str(dest), 4)

    src = nb.cells[0].source.splitlines()
    assert src[0] == '## Overloaded'
    assert src[1] == '## Foo'



import argparse

import nbformat
from nbformat.v4 import new_notebook
from nbformat.v4.nbjson import from_dict

from .tex2cells import Tex2Cells


def get_tex2cells_subclass(fp, fname):
    """Return a subclass of Tex2Cells defined in the given file and filename.

    Parameters
    ----------

    fp: :file: Input file object.
    fname: :str: Filename of file.
    """
    ns = {}
    exec(compile(fp.read(), fname, 'exec'), ns)
    converter = None
    for k, v in ns.items():
        if type(v) is type and issubclass(v, Tex2Cells) \
           and v is not Tex2Cells:
            converter = v
            break
    return converter


def tex2ipy(code, cls=Tex2Cells):
    t2c = cls(code)
    cells = t2c.parse()
    livereveal = dict(
        transition='none',
        scroll=True,
        controls=True,
        slideNumber=True,
        help=True
    )
    md = dict(
        language='python',
        celltoolbar='Slideshow',
        livereveal=livereveal
    )
    nb = new_notebook(
        metadata=from_dict(md),
        cells=from_dict(cells)
    )
    return nb


def main(args=None):
    parser = argparse.ArgumentParser(
        "Convert LaTeX beamer slides to IPython notebooks + RISE"
    )
    parser.add_argument("input", nargs=1, help="Input file (.tex)")
    parser.add_argument("output", nargs=1, help="Output file (.ipynb)")
    parser.add_argument(
        "-c", "--converter", action="store", dest="converter", default='',
        help="Path to a Python file which defines a subclass of Tex2Cells."
    )
    args = parser.parse_args(args)
    converter = None
    if len(args.converter) > 0:
        with open(args.converter) as fp:
            converter = get_tex2cells_subclass(fp, args.converter)
    if converter is None:
        converter = Tex2Cells

    with open(args.input[0]) as f:
        code = f.read()
    nb = tex2ipy(code, converter)
    with open(args.output[0], 'w') as f:
        nbformat.write(nb, f)


if __name__ == '__main__':
    main()



from glob import glob
import os
import re

from TexSoup import TexSoup, TexNode


def get_all_listings(code):
    result = []
    start = (r'\begin{lstlisting}', r'\begin{verbatim}')
    end = (r'\end{lstlisting}', r'\end{verbatim}')
    in_listing = False
    for line in code.splitlines():
        llstrip = line.lstrip()
        if llstrip.startswith(start):
            in_listing = True
            block = []
            continue
        elif llstrip.endswith(end):
            in_listing = False
            result.append(block)
            continue
        elif in_listing:
            block.append(line + '\n')

    return result


def get_real_image_from_path(image_path):
    """Often images are provided without an extension, so we try to
    find a suitable one using glob.
    """
    formats = ('png', 'jpg', 'svg', 'gif', 'jpeg', 'bmp')
    image = image_path
    if not os.path.exists(image):
        for f in glob(image+'*'):
            if f.lower().endswith(formats):
                image = f
                break
    return image


def _replace_display_math(code):
    code = code.replace(r'\[', r'\begin{equation*}')
    return code.replace(r'\]', r'\end{equation*}')


def remove_comments(code):
    return re.sub('(?<!\\\\)%.*$', '', code, flags=re.M)


class Tex2Cells(object):
    def __init__(self, code):
        code = _replace_display_math(code)
        code = remove_comments(code)
        self.soup = TexSoup(code)
        self.listings = get_all_listings(code)
        self._listings_count = 0
        self.info = {}
        self.cells = []
        self.current = None

    def _parse_titlepage(self):
        nodes = ('title', 'author', 'institute', 'date', 'logo')
        for node in nodes:
            for elem in self.soup.find_all(node):
                method = getattr(self, '_handle_' + node)
                method(elem)

    def parse(self):
        """Parse the given TeX code and return suitable IPython cells.
        """
        self._parse_titlepage()
        doc = self.soup.find('document')
        self._walk(doc)
        return self.cells

    def _walk(self, node):
        if isinstance(node, TexNode):
            name = node.name.replace('*', '_star')
            method_name = '_handle_%s' % name
            method = getattr(self, method_name, None)
            skip_children = False
            if method:
                skip_children = method(node)
            else:
                self._handle_unknown(node)
            if not skip_children:
                for element in node.contents:
                    self._walk(element)
        elif isinstance(node, str):  # pragma: no branch
            if self.current is not None:  # pragma: no branch
                self._handle_str(node)

    def _make_cell(self, cell_type='markdown', slide_type='slide'):
        slideshow = dict(slide_type=slide_type)
        if cell_type == 'markdown':
            self.current = dict(
                cell_type=cell_type,
                metadata=dict(slideshow=slideshow),
                source=[]
            )
        else:
            self.current = dict(
                cell_type=cell_type,
                metadata=dict(slideshow=slideshow),
                source=[],
                outputs=[],
                execution_count=None
            )
        self.cells.append(self.current)

    def _clear_newline(self, s):
        if s.startswith(('\n', '\r')):
            return s.lstrip()
        else:
            return s

    def _handle_block(self, node):
        src = self.current['source']
        src.append('### %s\n' % node.args[0])
        src.append('')
        return False

    def _handle_equation(self, node):
        src = self.current['source']
        src.append('$$\n')
        src.append(''.join(self._clear_newline(str(x)) for x in node.contents))
        src.append('$$\n')
        return True

    _handle_equation_star = _handle_equation
    _handle_align = _handle_equation
    _handle_align_star = _handle_equation
    _handle_eqnarray = _handle_equation
    _handle_eqnarray_star = _handle_equation

    def _handle_emph(self, node):
        src = self.current['source']
        if len(src) == 0:
            src.append('')
        src[-1] += '*%s* ' % node.string
        return True

    def _handle_frame(self, node):
        self._make_cell(cell_type='markdown', slide_type='slide')

    def _handle_frametitle(self, node):
        src = self.current['source']
        src.append('## ')
        for item in node.contents:
            self._walk(item)

        src[-1] += '\n'
        src.append('')

        return True

    def _handle_str(self, node):
        src = self.current['source']
        data = self._clear_newline(str(node))
        if len(src) == 0:
            src.append('')

        src[-1] += data

    def _handle_item(self, node):
        src = self.current['source']
        if len(src) > 0:
            src[-1] += '\n'
        if node.parent.name == 'itemize':
            src.append('* ')
        elif node.parent.name == 'enumerate':  # pragma: no branch
            src.append('1. ')
        else:  # pragma: no cover
            print(r"\item has unknown parent node", node.parent.name)

    def _handle_itemize(self, node):
        for item in node.contents:
            self._walk(item)
        src = self.current['source']
        if len(src) > 0:  # pragma: no branch
            if not src[-1].endswith('\n'):
                src[-1] += '\n'
        return True

    _handle_enumerate = _handle_itemize

    def _handle_lstlisting(self, node):
        cell = self.current
        if cell is not None and len(cell['source']) == 0:
            # Empty cell already exists, just set its type.
            cell['cell_type'] = 'code'
            cell['execution_count'] = None
            cell['outputs'] = []
        else:
            self._make_cell(cell_type='code', slide_type='-')
        del cell
        src = []
        code = self.listings[self._listings_count]
        START = ('In []:', '...:', '....:', '.....:')
        for line in code:
            llstrip = line.lstrip()
            if llstrip.startswith(START):
                idx = line.index(':') + 2
                src.append(line[idx:])
            elif llstrip.startswith('Out[]:'):
                self.current['source'] = src
                self._make_cell(cell_type='code', slide_type='-')
                src = []
            else:
                src.append(line)
        if len(src) > 0:
            self.current['source'] = src

        self._listings_count += 1

        # If there is an itemize after the lstlisting, it will add source
        # elements into the code cell.
        siblings = list(node.parent.contents)
        s_repr = list(repr(x) for x in siblings)
        index = s_repr.index(repr(node))
        if index < (len(siblings) - 1):
            self._make_cell(slide_type='-')
        return True

    _handle_verbatim = _handle_lstlisting

    def _handle_ldots(self, node):
        self._handle_str('...')
        return True

    def _handle_pause(self, node):
        src = self.current['source']
        if len(src) == 0:
            # No content in the current cell.
            self.current['metadata']['slideshow']['slide_type'] = 'fragment'
        else:
            self._make_cell(slide_type='fragment')

    def _handle_textbf(self, node):
        src = self.current['source']
        if len(src) == 0:
            src.append('')

        src[-1] += '**%s** ' % node.string
        return True

    def _handle_texttt(self, node):
        src = self.current['source']
        if len(src) == 0:
            src.append('')
        src[-1] += '`%s` ' % node.string
        return True

    _handle_lstinline = _handle_texttt
    _handle_py = _handle_lstinline
    _handle_PythonCode = _handle_lstinline

    def _handle_title(self, node):
        contents = list(node.contents)
        if contents:
            self.info[node.name] = contents[-1]
        return True

    _handle_author = _handle_title
    _handle_institute = _handle_title
    _handle_date = _handle_title
    _handle_logo = _handle_title

    def _handle_titlepage(self, node):
        src = self.current['source']
        src.append('# %s\n' % self.info.get('title', 'Title'))
        src.append('\n')
        src.append('**%s**\n' % self.info.get('author', 'Author'))
        src.append('\n')
        src.append('**%s**\n' % self.info.get('institute', 'Institute'))
        src.append('\n')
        src.append('**%s**\n' % self.info.get('date', 'Date'))
        src.append('\n')

    def _handle_pgfimage(self, node):
        src = self.current['source']
        data = list(node.contents)
        image = get_real_image_from_path(data[-1])
        src.append('<img src="%s"/>\n' % image)
        return True

    _handle_includegraphics = _handle_pgfimage

    def _handle_movie(self, node):
        src = self.current['source']
        data = list(node.contents)
        movie = data[-1]
        src.append('<div align="center">\n')
        src.append('<video loop controls src="%s"/>\n' % movie)
        src.append('</div>\n')
        return True

    def _handle_url(self, node):
        src = self.current['source']
        src.append('<%s> ' % node.string)
        return True

    def _handle_quote(self, node):
        src = self.current['source']
        for line in node.contents:
            for l in line.lstrip().splitlines():
                src.append('> %s\n' % l)
        return True

    def _handle_document(self, node):
        self.cells = []
        self.current = None

    def _handle_hrule(self, node):
        self.current['source'].append('\n----\n')

    def _handle_section(self, node):
        self._make_cell()
        self.current['source'].append('## %s\n' % node.string)
        return True

    def _handle_subsection(self, node):
        self._make_cell()
        self.current['source'].append('### %s\n' % node.string)
        return True

    def _handle_subsubsection(self, node):
        self._make_cell()
        self.current['source'].append('#### %s\n' % node.string)
        return True

    def _ignore(self, node):
        return False

    _handle_center = _ignore
    _handle_figure = _ignore
    _handle_minipage = _ignore
    _handle_centering = _ignore
    _handle_tiny = _ignore
    _handle_footnotesize = _ignore
    _handle_small = _ignore
    _handle_large = _ignore
    _handle_Large = _ignore
    _handle_huge = _ignore
    _handle_Huge = _ignore

    def _ignore_children(self, node):
        return True

    _handle_vspace = _ignore_children
    _handle_hspace = _ignore_children
    _handle_vspace_star = _ignore_children
    _handle_hspace_star = _ignore_children

    def _handle_unknown(self, node):
        src = self.current['source']
        if len(src) == 0:
            src.append('')

        src[-1] += '\\%s ' % node.name
        print("No handler for ", node.name)

    ####################################################################
    # The following are not generic LaTeX commands but specific to
    # the author's macros.
    _handle_media = _handle_movie

    def _handle_BackgroundPictureWidth(self, node):
        data = list(node.contents)
        if os.path.basename(data[-1]) in ('blank', 'blank.png'):
            return True
        self._make_cell(slide_type='slide')
        src = self.current['source']
        image = get_real_image_from_path(data[-1])
        src.append('<img width="100%%" src="%s"/>\n' % image)
        return True

    def _handle_BackgroundPictureHeight(self, node):
        data = list(node.contents)
        if os.path.basename(data[-1]) in ('blank', 'blank.png'):
            return True
        self._make_cell(slide_type='slide')
        src = self.current['source']
        image = get_real_image_from_path(data[-1])
        src.append('<img height="100%%" src="%s"/>\n' % image)
        return True

    _handle_BackgroundPicture = _handle_BackgroundPictureWidth
    _handle_typ = _handle_lstinline
    _handle_kwrd = _handle_lstinline



import os
from textwrap import dedent

from tex2ipy.tex2cells import Tex2Cells, get_all_listings, \
    get_real_image_from_path, remove_comments


def test_get_all_listings():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{lstlisting}
    In []: 1
    Out[]: 1
    In []: print 1
    \end{lstlisting}

    \begin{itemize}
    \item blah
    \end{itemize}

    \begin{verbatim}
    In []: 2
    Out[]: 2
    In []: print 2
    \end{verbatim}

    \end{document}
    """)

    # When
    lst = get_all_listings(doc)

    # Then
    assert len(lst) == 2
    expect = ['In []: 1\n', 'Out[]: 1\n', 'In []: print 1\n']
    assert lst[0] == expect

    expect = ['In []: 2\n', 'Out[]: 2\n', 'In []: print 2\n']
    assert lst[1] == expect


def test_get_real_image_from_path(tmpdir):
    # Given a path that exists, check if that image path is given.
    img = tmpdir.join('image.png')
    img.write('')
    img_path = str(img)
    image = get_real_image_from_path(img_path)
    assert image == img_path
    img.remove()

    for ext in ('.png', '.PNG', '.jpg', '.JPEG', '.SVG', '.gif', '.BMP'):
        img = tmpdir.join('image' + ext)
        img.write('')
        img_path = str(img)
        path = os.path.splitext(img_path)[0]
        image = get_real_image_from_path(path)
        assert image == img_path
        img.remove()

    # Case when the image doesn't exist and it is not supported.
    img = tmpdir.join('image.xpm')
    img.write('')
    img_path = str(img)
    path = os.path.splitext(img_path)[0]
    image = get_real_image_from_path(path)
    assert image == path
    img.remove()


def test_remove_comments():
    # Given
    doc = dedent(r"""
    hello %world
    world % \alpha_beta
    % \item foo
    \emph{\%s} is correct
    """)

    # When
    doc = remove_comments(doc)

    # Then
    expect = '\nhello \nworld \n\n\emph{\\%s} is correct\n'
    assert doc == expect


def test_lstlisting_with_output_should_make_multiple_cells():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{lstlisting}
    In []: 1
    Out[]: 1
    In []: 1+1
    Out[]: 2
    \end{lstlisting}
    \end{document}
    """)
    t2c = Tex2Cells(doc)

    # When
    cells = t2c.parse()

    # Then
    assert len(cells) == 3
    assert cells[0]['source'] == ['1\n']
    assert cells[0]['cell_type'] == 'code'
    assert cells[0]['metadata']['slideshow']['slide_type'] == '-'
    assert cells[1]['source'] == ['1+1\n']
    assert cells[1]['cell_type'] == 'code'
    assert cells[1]['metadata']['slideshow']['slide_type'] == '-'


def test_multiple_lstlistings():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \begin{lstlisting}
    In []: 1
    \end{lstlisting}
    \begin{itemize}
    \item blah
    \end{itemize}
    \begin{verbatim}
    In []: 2
    print(2)
    >>> print('hello')
    \end{verbatim}
    \end{frame}
    \end{document}
    """)
    t2c = Tex2Cells(doc)

    # When
    cells = t2c.parse()

    # Then
    assert len(cells) == 3
    assert cells[0]['source'] == ['1\n']
    assert cells[0]['cell_type'] == 'code'
    assert cells[0]['metadata']['slideshow']['slide_type'] == 'slide'

    assert cells[1]['source'] == ['* blah\n']
    assert cells[1]['cell_type'] == 'markdown'
    assert cells[1]['metadata']['slideshow']['slide_type'] == '-'

    assert cells[2]['source'] == ['2\n', 'print(2)\n', ">>> print('hello')\n"]
    assert cells[2]['cell_type'] == 'code'
    assert cells[2]['metadata']['slideshow']['slide_type'] == '-'


def test_lstlistings_inside_itemize():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \begin{itemize}
    \item item1
    \begin{lstlisting}
    In []: 1
    \end{lstlisting}
    \item item2
    \begin{verbatim}
    print(2)
    \end{verbatim}
    \end{itemize}
    \end{frame}
    \end{document}
    """)
    t2c = Tex2Cells(doc)

    # When
    cells = t2c.parse()

    # Then
    assert len(cells) == 4
    assert cells[0]['source'] == ['* item1']
    assert cells[0]['cell_type'] == 'markdown'
    assert cells[0]['metadata']['slideshow']['slide_type'] == 'slide'

    assert cells[1]['source'] == ['1\n']
    assert cells[1]['cell_type'] == 'code'
    assert cells[1]['metadata']['slideshow']['slide_type'] == '-'

    assert cells[2]['source'] == ['* item2']
    assert cells[2]['cell_type'] == 'markdown'
    assert cells[2]['metadata']['slideshow']['slide_type'] == '-'

    assert cells[3]['source'] == ['print(2)\n']
    assert cells[3]['cell_type'] == 'code'
    assert cells[3]['metadata']['slideshow']['slide_type'] == '-'


def test_titlepage_is_created():
    # Given
    doc = dedent(r"""
    \documentclass[14pt, compress]{beamer}

    \begin{document}
    \title{foo}
    \author{blah}
    \date{date}
    \institute{Institute}

    \begin{frame}
    \titlepage
    \end{frame}

    \begin{frame}
    \frametitle{Foo}
    Hello world
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 2
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    assert src[0] == '# foo\n'
    assert src[1] == '\n'
    assert src[2] == '**blah**\n'
    assert src[3] == '\n'
    assert src[4] == '**Institute**\n'
    assert src[5] == '\n'
    assert src[6] == '**date**\n'

    assert cells[1]['cell_type'] == 'markdown'
    src = cells[1]['source']
    assert src[0] == "## Foo\n"
    assert src[1] == "Hello world\n"


def test_titlepage_is_created_when_title_is_outside_document():
    # Given
    doc = dedent(r"""
    \documentclass[14pt, compress]{beamer}
    \title{foo}
    \author{blah}
    \date{date}
    \begin{document}
    \begin{frame}
    \titlepage
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    assert src[0] == '# foo\n'
    assert src[1] == '\n'
    assert src[2] == '**blah**\n'
    assert src[3] == '\n'
    assert src[4] == '**Institute**\n'
    assert src[5] == '\n'
    assert src[6] == '**date**\n'


def test_itemize_enumerate_works():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \frametitle{Title}
       \begin{itemize}
        \item item 1
        \item item 2
       \end{itemize}
       \begin{enumerate}
        \item aa
        \item bb
       \end{enumerate}
    hello
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    print(src)
    assert src[0] == '## Title\n'
    assert src[1] == '\n'
    assert src[2] == '* item 1\n'
    assert src[3] == '* item 2\n\n'
    assert src[4] == '1. aa\n'
    assert src[5] == '1. bb\nhello\n'


def test_images_should_work_figure_ignored():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \frametitle{Title}
    \begin{figure}
    \pgfimage[height=1cm,width=1cm]{images/img1.png}
    \end{figure}
    \includegraphics[height=1cm,width=1cm]{images/img2.jpg}
    \url{www.python.org}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    print(src)
    assert src[0] == '## Title\n'
    assert src[1] == ''
    assert src[2] == '<img src="images/img1.png"/>\n'
    assert src[3] == '<img src="images/img2.jpg"/>\n'
    assert src[4] == '<www.python.org> '


def test_movie_env_works():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \movie[width=9cm, height=5cm]{}{movies/movie.mp4}
    \media{poster.png}{movies/movie.mp4}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    print(src)
    assert src[0] == '<div align="center">\n'
    assert src[1] == '<video loop controls src="movies/movie.mp4"/>\n'
    assert src[2] == '</div>\n'
    assert src[3] == '<div align="center">\n'
    assert src[4] == '<video loop controls src="movies/movie.mp4"/>\n'
    assert src[5] == '</div>\n'


def test_space_should_be_ignored():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \frametitle{Title}
    \hspace{0.1in} hello
    \hspace*{0.1in} world
    \vspace{0.1in} hola
    \vspace*{0.1in} namaste
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    assert len(src) == 2
    print(src)
    assert src[0] == '## Title\n'
    assert src[1] == ' hello\n world\n hola\n namaste\n'


def test_quote_works_center_ignored_hrule_works():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \begin{center}
    \begin{quote}
    To do
    or not to do.
    \end{quote}
    \hrule
    \end{center}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    assert len(src) == 3
    print(src)
    assert src[0] == '> To do\n'
    assert src[1] == '> or not to do.\n'
    assert src[2] == '\n----\n'


def test_document_sections():
    # Given
    doc = dedent(r"""
    \begin{document}
    \section{Introduction}
    \subsection{Motivation}
    \subsubsection{Blah}
    \section{Methods}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 4
    assert cells[0]['cell_type'] == 'markdown'
    assert cells[0]['source'] == ['## Introduction\n']
    assert cells[1]['cell_type'] == 'markdown'
    assert cells[1]['source'] == ['### Motivation\n']
    assert cells[2]['cell_type'] == 'markdown'
    assert cells[2]['source'] == ['#### Blah\n']
    assert cells[3]['cell_type'] == 'markdown'
    assert cells[3]['source'] == ['## Methods\n']


def test_minipage_is_ignored():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \begin{minipage}
    hello world
    \end{minipage}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    assert cells[0]['source'] == ['hello world\n']


def test_inline_equations():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    $\int f(x) dx$
    \begin{itemize}
    \item hello $\alpha + \frac{1}{2} \beta_{\gamma}$ world
    \end{itemize}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    print(src)
    expect = [
        r'$\int f(x) dx$' + '\n',
        r'* hello$\alpha + \frac{1}{2} \beta_{\gamma}$ world' + '\n'
    ]
    assert src == expect


def test_equations():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \begin{equation}
    \alpha + \frac{1}{2} \beta_1
    \end{equation}
    \begin{equation*}
    \beta = \gamma
    \end{equation*}
    \begin{align}
    \beta = \gamma
    \end{align}
    \begin{align*}
    \beta = \gamma
    \end{align*}
    \[v_\theta = \frac{\Gamma}{2\pi r}\]
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    print(src)
    assert src[0] == '$$\n'
    assert src[1] == r'\alpha + \frac{1}{2} \beta_1' + '\n'
    assert src[2] == '$$\n'
    assert src[3] == '$$\n'
    assert src[4] == r'\beta = \gamma'
    assert src[5] == '$$\n'
    assert src[6] == '$$\n'
    assert src[7] == r'\beta = \gamma'
    assert src[8] == '$$\n'
    assert src[9] == '$$\n'
    assert src[10] == r'\beta = \gamma'
    assert src[11] == '$$\n'
    assert src[12] == '$$\n'
    assert src[13] == r'v_\theta = \frac{\Gamma}{2\pi r}'
    assert src[14] == '$$\n'


def test_pause_should_add_new_fragment():
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \begin{itemize}
    \item item 1
    \pause
    \item item 2
    \end{itemize}
    \begin{lstlisting}
    print 1
    \end{lstlisting}
    \pause
    \begin{lstlisting}
    print 2
    \end{lstlisting}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    print(cells)
    assert len(cells) == 4
    assert cells[0]['cell_type'] == 'markdown'
    assert cells[0]['metadata']['slideshow']['slide_type'] == 'slide'
    src = cells[0]['source']
    assert src[0] == '* item 1'

    assert cells[1]['cell_type'] == 'markdown'
    assert cells[1]['metadata']['slideshow']['slide_type'] == 'fragment'
    src = cells[1]['source']
    assert src[0] == '* item 2\n'

    assert cells[2]['cell_type'] == 'code'
    assert cells[2]['metadata']['slideshow']['slide_type'] == '-'
    src = cells[2]['source']
    assert src[0] == 'print 1\n'

    assert cells[3]['cell_type'] == 'code'
    assert cells[3]['metadata']['slideshow']['slide_type'] == 'fragment'
    src = cells[3]['source']
    assert src[0] == 'print 2\n'


def test_background_picture():
    # Given
    doc = dedent(r"""
    \begin{document}
    \BackgroundPicture{images/img1.png}
    \BackgroundPicture{images/blank}
    \BackgroundPictureWidth{images/img2.jpg}
    \BackgroundPictureWidth{images/blank}
    \BackgroundPictureHeight{images/img3.jpg}
    \BackgroundPictureHeight{images/blank}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 3

    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    assert len(src) == 1
    assert src[0] == '<img width="100%" src="images/img1.png"/>\n'

    assert cells[1]['cell_type'] == 'markdown'
    src = cells[1]['source']
    assert len(src) == 1
    assert src[0] == '<img width="100%" src="images/img2.jpg"/>\n'

    assert cells[2]['cell_type'] == 'markdown'
    src = cells[2]['source']
    assert len(src) == 1
    assert src[0] == '<img height="100%" src="images/img3.jpg"/>\n'


def test_text_embellishments():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \textbf{bold} \emph{emph} \texttt{texttt} \lstinline{code}
    \typ{code} \kwrd{for} hello \ldots
    \end{frame}
    \begin{frame}
    \emph{emph}
    \end{frame}
    \begin{frame}
    \texttt{texttt}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 3
    src = cells[0]['source']
    assert src[0] == '**bold** *emph* `texttt` `code` `code` `for`  hello ...'
    src = cells[1]['source']
    assert src[0] == '*emph* '
    src = cells[2]['source']
    assert src[0] == '`texttt` '


def test_unknown_macro_should_be_in_source():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \something
    \other{hello}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    src = cells[0]['source']
    assert src == ['\\something \\other hello']


def test_block_is_handled():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \begin{block}{Test block}
    Hello world
    \end{block}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    src = cells[0]['source']
    assert src[0] == '### Test block\n'
    assert src[1] == 'Hello world\n'


def test_title_can_have_text_embellishments():
    # Given
    doc = dedent(r"""
    \begin{document}
    \begin{frame}
    \frametitle{\emph{emph} \textbf{bold} $\alpha$  \lstinline{print}}
    \end{frame}
    \end{document}
    """)

    # When
    t2c = Tex2Cells(doc)
    cells = t2c.parse()

    # Then
    assert len(cells) == 1
    assert cells[0]['cell_type'] == 'markdown'
    src = cells[0]['source']
    print(src)
    assert src[0] == '## *emph*  **bold**  $\\alpha$  `print` \n'



from setuptools import setup, find_packages


def get_readme_rst():
    import subprocess
    try:
        proc = subprocess.Popen(
            ['pandoc', 'README.md', '--to', 'rst'],
            stdout=subprocess.PIPE
        )
        description = proc.communicate()[0].decode()
    except OSError:
        with open('README.md') as fp:
            description = fp.read()
    return description


def get_version():
    import os
    data = {}
    fname = os.path.join('tex2ipy', '__init__.py')
    exec(compile(open(fname).read(), fname, 'exec'), data)
    return data.get('__version__')


install_requires = ['TexSoup', 'nbformat']
tests_require = ['pytest']


classes = """
Development Status :: 3 - Alpha
Environment :: Console
Framework :: IPython
Intended Audience :: Developers
Intended Audience :: Education
Intended Audience :: End Users/Desktop
Intended Audience :: Science/Research
License :: OSI Approved :: BSD License
Natural Language :: English
Operating System :: OS Independent
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Python :: 3.3
Programming Language :: Python :: 3.4
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Programming Language :: Python :: 3 :: Only
Topic :: Software Development :: Libraries
Topic :: Utilities
"""

classifiers = [x.strip() for x in classes.splitlines() if x]

setup(
    name='tex2ipy',
    version=get_version(),
    author='Prabhu Ramachandran',
    author_email='prabhu@aero.iitb.ac.in',
    description='Convert beamer presentations to IPython notebooks',
    long_description=get_readme_rst(),
    license="BSD",
    url='https://github.com/prabhuramachandran/tex2ipy',
    classifiers=classifiers,
    packages=find_packages(),
    install_requires=install_requires,
    tests_require=tests_require,
    package_dir={'tex2ipy': 'tex2ipy'},
    entry_points="""
        [console_scripts]
        tex2ipy = tex2ipy.cli:main
    """
)
